#include <QApplication>
#include <QWidget>
#include <QDebug>
#include <QMouseEvent>
#include <QTimer>
#if QT_VERSION >= 0x050000
    #include <QWindow>
#endif

class EventFilter : public QObject
{
public:
    explicit EventFilter(QObject *parent = 0) : QObject(parent) {}

    bool eventFilter(QObject *watched, QEvent *event) override;
};

static void formatEvent(QObject *watched, QEvent *event)
{
    static int n = 0;
    QDebug d = qDebug();
#if QT_VERSION >= 0x050000
    d.noquote();
#endif
    d.nospace();
    d << '#' << n++ << ' ';
    QString on = watched->objectName();
    if (!on.isEmpty())
        d << '"' << on << "\"/";
    d << watched->metaObject()->className() << ' ' << event;
    if (watched->isWidgetType())
        d << " " << static_cast<const QWidget *>(watched)->windowState();
#if QT_VERSION >= 0x050000
    if (watched->isWindowType())
        d << " " << static_cast<const QWindow *>(watched)->windowStates();
#endif
}

bool EventFilter::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type()) {
    case QEvent::WindowStateChange:
        formatEvent(watched, event);
        break;
    default:
        break;
    }
   return QObject::eventFilter(watched, event);
}

class Parent : public QWidget
{
    Q_OBJECT
public:
    Parent()
    {
        connect(&m_timer, SIGNAL(timeout()), this, SLOT(slotTimer()));
        m_timer.start(1000);
    }

public slots:
    void slotTimer();

private:
    QTimer m_timer;
    int m_tick = 0;
};

void Parent::slotTimer()
{
    switch (m_tick++) {
    case 0:
        qDebug(">parent.setWindowState(Qt::WindowMinimized);");
        setWindowState(Qt::WindowMinimized);
        break;
    case 1:
        qDebug(">parent.setWindowState(Qt::WindowMaximized);");
        setWindowState(Qt::WindowMaximized);
        break;
    case 10:
        QCoreApplication::quit();
        break;
    }
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QStringList args = a.arguments();
    args.pop_front();
    if (!args.contains("-e"))
        a.installEventFilter(new EventFilter(&a));

    Parent parent;
    parent.setObjectName("Parent");
    qDebug() << QT_VERSION_STR;
#if QT_VERSION >= 0x050000
    qDebug() << a.platformName();
#endif
    QWidget child(&parent, Qt::Window);

    parent.setWindowTitle("Qt " QT_VERSION_STR " Parent window");
    child.setWindowTitle("Qt " QT_VERSION_STR " Child window");
    child.setObjectName("Child");

    parent.showMaximized();
    child.showMaximized();

    qDebug(">exec()");
    int r = a.exec();
    qDebug(">exit");
    return r;
}

#include "main.moc"
